@@ -33,7 +33,10 @@ const MOUSE_POLL_FREQUENCY = 50;
 const CROSSHAIRS_CLIP_SIZE = [100, 100];
 
 // Settings
-const SHOW_KEY                  = 'show-magnifier';
+const APPLICATIONS_SCHEMA       = 'org.gnome.desktop.a11y.applications';
+const SHOW_KEY                  = 'screen-magnifier-enabled';
+
+const MAGNIFIER_SCHEMA          = 'org.gnome.desktop.a11y.magnifier';
 const SCREEN_POSITION_KEY       = 'screen-position';
 const MAG_FACTOR_KEY            = 'mag-factor';
 const LENS_MODE_KEY             = 'lens-mode';
@@ -268,7 +271,7 @@ Magnifier.prototype = {
 
         let thickness = this._settings.get_int(CROSS_HAIRS_THICKNESS_KEY);
         let color = this._settings.get_string(CROSS_HAIRS_COLOR_KEY);
-        let opacity = this._settings.get_int(CROSS_HAIRS_OPACITY_KEY);
+        let opacity = this._settings.get_double(CROSS_HAIRS_OPACITY_KEY);
         let length = this._settings.get_int(CROSS_HAIRS_LENGTH_KEY);
         let clip = this._settings.get_boolean(CROSS_HAIRS_CLIP_KEY);
 
@@ -354,22 +357,22 @@ Magnifier.prototype = {
 
     /**
      * setCrosshairsOpacity:
-     * @opacity:    Value between 0 (transparent) and 255 (fully opaque).
+     * @opacity:    Value between 0.0 (transparent) and 1.0 (fully opaque).
      */
     setCrosshairsOpacity: function(opacity) {
         if (this._crossHairs)
-            this._crossHairs.setOpacity(opacity);
+            this._crossHairs.setOpacity(opacity * 255);
     },
 
     /**
      * getCrosshairsOpacity:
-     * @return:     Value between 0 (transparent) and 255 (fully opaque).
+     * @return:     Value between 0.0 (transparent) and 1.0 (fully opaque).
      */
     getCrosshairsOpacity: function() {
         if (this._crossHairs)
-            return this._crossHairs.getOpacity();
+            return this._crossHairs.getOpacity() / 255.0;
         else
-            return 0;
+            return 0.0;
     },
 
     /**
@@ -438,7 +441,8 @@ Magnifier.prototype = {
     },
 
     _settingsInit: function(zoomRegion) {
-        this._settings = new Gio.Settings({ schema: 'org.gnome.accessibility.magnifier' });
+        this._appSettings = new Gio.Settings({ schema: APPLICATIONS_SCHEMA });
+        this._settings = new Gio.Settings({ schema: MAGNIFIER_SCHEMA });
 
         if (zoomRegion) {
             // Mag factor is accurate to two decimal places.
@@ -462,9 +466,9 @@ Magnifier.prototype = {
         this.addCrosshairs();
         this.setCrosshairsVisible(showCrosshairs);
 
-        this._settings.connect('changed::' + SHOW_KEY,
-                               Lang.bind(this, function() {
-            this.setActive(this._settings.get_boolean(SHOW_KEY));
+        this._appSettings.connect('changed::' + SHOW_KEY,
+                                  Lang.bind(this, function() {
+            this.setActive(this._appSettings.get_boolean(SHOW_KEY));
         }));
 
         this._settings.connect('changed::' + SCREEN_POSITION_KEY,
@@ -495,7 +499,7 @@ Magnifier.prototype = {
 
         this._settings.connect('changed::' + CROSS_HAIRS_OPACITY_KEY,
                                Lang.bind(this, function() {
-            this.setCrosshairsOpacity(this._settings.get_int(CROSS_HAIRS_OPACITY_KEY));
+            this.setCrosshairsOpacity(this._settings.get_double(CROSS_HAIRS_OPACITY_KEY));
         }));
 
         this._settings.connect('changed::' + CROSS_HAIRS_LENGTH_KEY,
@@ -508,7 +512,7 @@ Magnifier.prototype = {
             this.setCrosshairsClip(this._settings.get_boolean(CROSS_HAIRS_CLIP_KEY));
         }));
 
-        return this._settings.get_boolean(SHOW_KEY);
+        return this._appSettings.get_boolean(SHOW_KEY);
    },
 
     _updateScreenPosition: function() {
