@@ -189,9 +189,11 @@ NotificationDaemon.prototype = {
                      actions, hints, timeout) {
         let id;
 
-        // Filter out notifications from Empathy, since we
+        // Filter out chat and presence notifications from Empathy, since we
         // handle that information from telepathyClient.js
-        if (appName == 'Empathy') {
+        if (appName == 'Empathy' && (hints['category'] == 'im.received' ||
+              hints['category'] == 'presence.online' ||
+              hints['category'] == 'presence.offline')) {
             // Ignore replacesId since we already sent back a
             // NotificationClosed for that id.
             id = nextNotificationId++;
@@ -465,6 +467,14 @@ Source.prototype = {
         if (event.type() != Clutter.EventType.BUTTON_RELEASE)
             return false;
 
+        // Left clicks are passed through only where there aren't unacknowledged
+        // notifications, so it possible to open them in summary mode; right
+        // clicks are always forwarded, as the right click menu is not useful for
+        // tray icons
+        if (event.get_button() == 1 &&
+            this.notifications.length > 0)
+            return false;
+
         if (Main.overview.visible) {
             // We can't just connect to Main.overview's 'hidden' signal,
             // because it's emitted *before* it calls popModal()...
@@ -519,10 +529,13 @@ Source.prototype = {
     _appStateChanged: function() {
         // Destroy notification sources when their apps exit.
         // The app exiting would normally result in a tray icon being removed,
-        // so it should be ok to destroy the source associated with a tray icon
-        // here too, however we just let that happen through the code path
-        // associated with the tray icon being removed.
-        if (!this._isTrayIcon && this.app.get_state() == Shell.AppState.STOPPED)
+        // so the associated source would be destroyed through the code path
+        // that handles the tray icon being removed. We should not destroy
+        // the source associated with a tray icon when the application state
+        // is Shell.AppState.STOPPED because running applications that have
+        // no open windows would also have that state. This is often the case
+        // for applications that use tray icons.
+        if (!this._trayIcon && this.app.get_state() == Shell.AppState.STOPPED)
             this.destroy();
     },
 
